﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DateDiff.DateDiff
{
    /// <summary>
    /// Date difference calculation in three steps:
    /// - Calculate the remaing days in the preceding year.
    /// - Count the years between both dates and get the number of days.
    /// - Calculate the days in the last year until subsequent days.
    /// </summary>
    class DateDiffDirectThreeSteps : IDateDiff
    {
        private static readonly int[] daysUpToMonth = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };
        private static readonly int[] daysUpToMonthLeapYear = { 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335 };

        public DateDiffDirectThreeSteps(int y1, int m1, int d1, int y2, int m2, int d2) :
            base(y1, m1, d1, y2, m2, d2)
        {
            /* empty, see base constructor */
        }

        public override int GetDifference()
        {
            if (yearPreceding == yearSubsequent)
            {   /* same year, easy case */
                return GetDiffInDaysInSameYear(monthPreceding, dayPreceding, monthSubsequent, daySubsequent, DateDiffUtil.IsLeapYear(yearPreceding));
            }
            else
            {
                int resultDays = 0;

                /* handle first year which might be not a whole year */
                resultDays = GetRemainingDaysInYear(monthPreceding, dayPreceding, DateDiffUtil.IsLeapYear(yearPreceding));
                /* first year is now handled */
                yearPreceding++;

                while (yearPreceding < yearSubsequent)
                {
                    resultDays += DateDiffUtil.IsLeapYear(yearPreceding) ? 366 : 365;
                    yearPreceding++;
                }

                resultDays += GetDaysInYearTillDate(monthSubsequent, daySubsequent, DateDiffUtil.IsLeapYear(yearSubsequent));
                return resultDays * (IsDate1AfterDate2 ? -1 : 1);
            }
        }

        /// <summary>
        /// Get the difference in days between both dates, specified by day and month.
        /// Precondition: It is assumed that both dates lie in the same year.
        /// If date 2 lies before date 1, the result is negative.
        /// </summary>
        /// <param name="month1"></param>
        /// <param name="day1"></param>
        /// <param name="month2"></param>
        /// <param name="day2"></param>
        /// <returns></returns>
        public int GetDiffInDaysInSameYear(int month1, int day1, int month2, int day2, bool isLeapYear)
        {
            if (isLeapYear)
            {
                return daysUpToMonthLeapYear[month2 - 1] - daysUpToMonthLeapYear[month1 - 1] +
                      (day2 - day1);
            }
            else
            {
                return daysUpToMonth[month2 - 1] - daysUpToMonth[month1 - 1] +
                      (day2 - day1);
            }
        }

        /// <summary>
        /// Get the number of remaining days in given year, that is from month-day until 12-31.
        /// </summary>
        /// <param name="month"></param>
        /// <param name="day"></param>
        /// <param name="isLeapYear"></param>
        /// <returns></returns>
        public int GetRemainingDaysInYear(int month, int day, bool isLeapYear)
        {
            if (isLeapYear)
            {
                return 366 - (daysUpToMonthLeapYear[month - 1] + day);
            }
            else
            {
                return 365 - (daysUpToMonth[month - 1] + day);
            }
        }

        /// <summary>
        /// Get the number of days of given date, that is from 01-01 to month-day.
        /// </summary>
        /// <param name="month"></param>
        /// <param name="day"></param>
        /// <param name="isLeapYear"></param>
        /// <returns></returns>
        public int GetDaysInYearTillDate(int month, int day, bool isLeapYear)
        {
            if (isLeapYear)
            {
                return daysUpToMonthLeapYear[month - 1] + day;
            }
            else
            {
                return daysUpToMonth[month - 1] + day;
            }
        }
    }
}
