﻿/*
 * Implementation of different alogrithms to calculate the difference of two dates in days.
 * 
 * Algorithms:
 * - Using DateTime class of .NET framework
 * - Counting algorithm by incrementing the previous date until it's equal to the subsequent date.
 * - Direct calculating in three steps.
 * - Calculating the delay in days of both dates to an origin, then get the difference of both delays.
 * 
 * Sunshine, November 2014
 * 
 * www.sunshine2k.de || www.bastian-molkenthin.de
 */ 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DateDiff.DateDiff;

namespace DateDiff
{
    class Program
    {
        enum EDateDiffAlgorithm { DotNetLib, Increment, DirectThreeSteps, OriginBased };

        struct DateDiffInDaysTestEntry
        {
            public int PrecedingYear { get; private set; }
            public int PrecedingMonth { get; private set; }
            public int PrecedingDay { get; private set; }
            public int SubsequentYear { get; private set; }
            public int SubsequentMonth { get; private set; }
            public int SubsequentDay { get; private set; }

            public int ActualDifference { get; private set; }

            public DateDiffInDaysTestEntry(int yp, int mp, int dp, int ys, int ms, int ds, int diff) : this()
            {
                PrecedingYear = yp;
                PrecedingMonth = mp;
                PrecedingDay = dp;
                SubsequentYear = ys;
                SubsequentMonth = ms;
                SubsequentDay = ds;
                ActualDifference = diff;
            }

            public override String ToString()
            {
                return PrecedingYear + "-" + PrecedingMonth + "-" + PrecedingDay + " -> " +
                       SubsequentYear + "-" + SubsequentMonth + "-" + SubsequentDay;
            }
        }

        static DateDiffInDaysTestEntry[] testdataEntries = new DateDiffInDaysTestEntry [] 
        { 
            new DateDiffInDaysTestEntry(1899, 1, 1, 1900, 1, 30,    394),
            new DateDiffInDaysTestEntry(1601, 1, 1, 1970, 1, 1,     134774),
            new DateDiffInDaysTestEntry(2011, 9, 12, 2011, 11, 27,  76),
            new DateDiffInDaysTestEntry(2000, 7, 18, 2000, 7, 20,   2),
            new DateDiffInDaysTestEntry(2000, 2, 28, 2000, 3, 1,    2),
            new DateDiffInDaysTestEntry(2004, 2, 28, 2004, 3, 1,    2),
            new DateDiffInDaysTestEntry(2006, 2, 28, 2006, 3, 1,    1),
            new DateDiffInDaysTestEntry(1900, 2, 28, 1900, 3, 1,    1),
            new DateDiffInDaysTestEntry(2012, 2, 28, 2016, 2, 29,   1462),
            new DateDiffInDaysTestEntry(2012, 2, 28, 2016, 2, 27,   1460),
            new DateDiffInDaysTestEntry(2000, 2, 27, 2004, 1, 1,   1404),
            new DateDiffInDaysTestEntry(2000, 2, 27, 2004, 5, 5,   1529),
            new DateDiffInDaysTestEntry(2000, 2, 27, 1999, 5, 5,   -298),
            new DateDiffInDaysTestEntry(2003, 8, 20, 1998, 10, 11,   -1774),
            new DateDiffInDaysTestEntry(1999, 5, 5, 2000, 2, 27,   298),
            new DateDiffInDaysTestEntry(1999, 5, 5, 1999, 12, 31,   240),
            new DateDiffInDaysTestEntry(2010, 2, 21, 2011, 1, 13, 326),
            new DateDiffInDaysTestEntry(1719, 2, 14, 1812, 4, 30, 34043)
        };

       

        private static void runTestsNew(EDateDiffAlgorithm dateDiffAlgorithm)
        {
            bool overallVerdictPassed = true;

            foreach (DateDiffInDaysTestEntry te in testdataEntries)
            {
                IDateDiff dateDiffDotnet = GetAlgorithmInstance(dateDiffAlgorithm, te);

                int diff = dateDiffDotnet.GetDifference();

                if (diff == te.ActualDifference)
                {
                    Console.Out.Write("x");
                }
                else
                {
                    Console.Out.WriteLine();
                    Console.Out.WriteLine("F: " + te + " Diff: " + diff + " (Exp.: " + te.ActualDifference + ")");
                    overallVerdictPassed = false;
                }
            }

            Console.Out.WriteLine();
            Console.Out.WriteLine(overallVerdictPassed ? "SUCCESS" : "FAILED");
        }



        private static IDateDiff GetAlgorithmInstance(EDateDiffAlgorithm algoType, DateDiffInDaysTestEntry te)
        {
            switch (algoType)
            {
                case EDateDiffAlgorithm.DotNetLib:
                    return new DateDiffDotnetLib(te.PrecedingYear, te.PrecedingMonth, te.PrecedingDay,
                                te.SubsequentYear, te.SubsequentMonth, te.SubsequentDay);

                case EDateDiffAlgorithm.Increment:
                    return new DateDiffByIncrement(te.PrecedingYear, te.PrecedingMonth, te.PrecedingDay,
                                te.SubsequentYear, te.SubsequentMonth, te.SubsequentDay);

                case EDateDiffAlgorithm.DirectThreeSteps:
                    return new DateDiffDirectThreeSteps(te.PrecedingYear, te.PrecedingMonth, te.PrecedingDay,
                                te.SubsequentYear, te.SubsequentMonth, te.SubsequentDay);

                case EDateDiffAlgorithm.OriginBased:
                    return new DateDiffByDiffToOrigin(te.PrecedingYear, te.PrecedingMonth, te.PrecedingDay,
                                te.SubsequentYear, te.SubsequentMonth, te.SubsequentDay);

                default: throw new ArgumentException("Invalid or not supported algotype");
            }
        }

        /// <summary>
        /// Entry point.
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            Console.Out.WriteLine("Starting tests ...");

            Console.Out.WriteLine("Using DateDiffDotnetLib2:");
            runTestsNew(EDateDiffAlgorithm.DotNetLib);
            Console.Out.WriteLine("------------------------------");

            Console.Out.WriteLine("Using DateDiffByIncrement2:");
            runTestsNew(EDateDiffAlgorithm.Increment);
            Console.Out.WriteLine("------------------------------");

            Console.Out.WriteLine("Using DateDiffDirect:");
            runTestsNew(EDateDiffAlgorithm.DirectThreeSteps);
            Console.Out.WriteLine("------------------------------");

            Console.Out.WriteLine("Using DateDiffByDiffToOrigin2:");
            runTestsNew(EDateDiffAlgorithm.OriginBased);
            Console.Out.WriteLine("------------------------------");

            Console.Out.WriteLine("Tests finished. Press any key to quit...");
            Console.In.ReadLine();
        }
    }
}
